# -*- coding: utf-8 -*-

from hypermap.dynasty.models import Dynasty
import re


def get_mined_dates(text):
    mined_dates = []
    dates = mine_date(text)
    if dates:
        for date in dates:
            if isinstance(date, list):
                for range_date in date:
                    mined_dates.append(range_date)
                    # TODO here we need to detect first date and last date
            else:
                mined_dates.append(date)
    # we remove duplicates
    mined_dates = list(set(mined_dates))
    return mined_dates


def clean_text(text):
    try:
        text = text.decode("utf-8")
    except UnicodeEncodeError:
        text = text.encode("ascii", "ignore")
    except:
        pass
    return text


def year_miner(text):
    dates = []
    try:
        years = re.findall('\d{2,4} ?B?CE', text)
        bc_years = re.findall('\d{2,4} ?BC', text)
    except:
        pass
    if bc_years:
        for bc_year in bc_years:
            if "BC" in bc_year:
                bc_year = re.findall('\d+', bc_year)[0]
                bcdate = str('-'+str(bc_year).zfill(4))+'-01'+'-01'
                dates.append(bcdate)
    if years:
        for year in years:
            if "CE" in year and "BCE" not in year:
                year = re.findall('\d+', year)[0]
                cedate = str(year.zfill(4))+'-01'+'-01'
                dates.append(cedate)
            if "BCE" in year and bc_years is None:
                year = re.findall('\d+', year)[0]
                bcedate = str('-'+str(year).zfill(4))+'-01'+'-01'
                dates.append(bcedate)
    return dates


def dynasty_miner(text):
    date_range = None
    dates = []
    dynasties = Dynasty.objects.values_list('name', flat=True)
    word_set = set(dynasties)
    text_set = set(text.split())
    common_set = None
    if word_set.intersection(text_set):
        common_set = word_set.intersection(text_set)
    if common_set:
        for item in common_set:
            date_range = Dynasty.objects.get(name=item).date_range
    if date_range:
        years = re.findall('[-\d]+', date_range)
        for year in years:
            if year.startswith('-'):
                year = year[1:]
                date = str('-'+str(year).zfill(4))+'-01'+'-01'
            else:
                date = str(str(year).zfill(4))+'-01'+'-01'
            dates.append(date)
    return dates


def valid_dates(years):
    valid_years = []
    for year in years:
        if len(year) <= 4 and int(year) >= 1400:
            date = str(year+'-01'+'-01')
            valid_years.append(date)
    return valid_years


def mine_date(text):
    text = clean_text(text)
    date = None
    dates = []
    try:
        years = re.findall(r'\d+', text)
    except:
        pass
    if dynasty_miner(text):
        dates.append(dynasty_miner(text))
    if year_miner(text):
        if date:
            dates.remove(date)
        dates.append(year_miner(text))
    if years and not year_miner(text):
        for years in valid_dates(years):
            dates.append(years)
    if dates:
        date = dates
    return date



# -*- coding: utf-8 -*-

"""
Tests for the WorldMap Service Type.
"""

import unittest

from httmock import with_httmock
import mocks.worldmap

from hypermap.aggregator.models import Service, Catalog
from hypermap.aggregator.enums import DATE_DETECTED, DATE_FROM_METADATA


class TestWorldMap(unittest.TestCase):

    @with_httmock(mocks.worldmap.resource_get)
    def test_create_worldmap_service(self):

        catalog, created = Catalog.objects.get_or_create(
            name="hypermap", slug="hypermap",
            url="search_api"
        )

        # create the service
        service = Service(
            type='Hypermap:WorldMap',
            url='http://worldmap.harvard.edu/',
            catalog=catalog
        )
        service.save()

        # check title is correct
        self.assertTrue(service.title, 'Harvard WorldMap')

        # check layer number
        self.assertEqual(service.layer_set.all().count(), 10)

        # check layer geonode:River_basin_num2 (public)
        layer_0 = service.layer_set.get(name='geonode:River_basin_num2')
        self.assertEqual(layer_0.title, 'China River Basins')
        self.assertTrue(layer_0.is_public)
        self.assertEqual(layer_0.layerwm.temporal_extent_start, '1971-02-06')
        self.assertEqual(layer_0.layerwm.temporal_extent_end, '1974-09-30')
        self.assertEqual(layer_0.layerwm.category, 'Rivers, Streams, Lakes')
        self.assertEqual(layer_0.layerwm.username, 'matt')
        self.assertEqual(layer_0.keywords.all().count(), 4)
        self.assertEqual(layer_0.service.srs.all().count(), 3)
        self.assertEqual(layer_0.check_set.all().count(), 1)

        # test dates #1
        layer_with_many_dates = service.layer_set.get(name='geonode:layer_with_many_dates')
        # this layer has the following dates
        # in title: 1999, 1882
        # in abstract: 1632, 1661, 1992, 1338
        # from metadata: temporal_extent_start: 2011-01-24
        # from metadata: temporal_extent_end: 2015-09-30
        # check detected dates
        for date in ('1999-01-01', '1882-01-01', '1632-01-01', '1661-01-01', '1992-01-01'):
            date_filtered = layer_with_many_dates.layerdate_set.filter(date=date)
            self.assertEqual(date_filtered.filter(type=DATE_DETECTED).count(), 1)
        # check metadata dates
        for date in ('2011-01-24', '2015-09-30'):
            date_filtered = layer_with_many_dates.layerdate_set.filter(date=date)
            self.assertEqual(
                             date_filtered.filter(type=DATE_FROM_METADATA).count(), 1)

        # test dates #2
        layer_with_few_dates = service.layer_set.get(name='geonode:layer_with_few_dates')
        # this layer has the following dates
        # in title: 1990
        # in abstract: 1990, 1996, 2000
        # from metadata: temporal_extent_start: 1990-01-01
        # from metadata: temporal_extent_end: Date in invalid format
        # check detected dates
        for date in ('1990-01-01', '1996-01-01', '2000-01-01'):
            self.assertEqual(layer_with_few_dates.layerdate_set.filter(date=date).filter(type=DATE_DETECTED).count(), 1)
        # check metadata dates
        for date in ('1990-01-01', ):

            date_filtered = layer_with_few_dates.layerdate_set.filter(date=date)
            self.assertEqual(date_filtered.filter(type=DATE_FROM_METADATA).count(), 1)

        # test dates #3
        layer_with_complex_dates = service.layer_set.get(name='geonode:layer_with_complex_dates')
        # this layer has the following dates
        # in abstract: 1900 BCE, 2000 BCE, Xia dynasty
        # from metadata: temporal_extent_start: -1900-01-01
        # from metadata: temporal_extent_end: -2000-01-01
        for date in ('-1900-01-01', '-2000-01-01', '-1600-01-01', '-2100-01-01'):
            date_filtered = layer_with_complex_dates.layerdate_set.filter(date=date)
            self.assertEqual(date_filtered.filter(type=DATE_DETECTED).count(), 1)
        # check metadata dates
        for date in ('-1900-01-01', '-2000-01-01', ):
            date_filtered = layer_with_complex_dates.layerdate_set.filter(date=date)
            self.assertEqual(date_filtered.filter(type=DATE_FROM_METADATA).count(), 1)

        # test dates #4
        layer_with_dates_in_abstract = service.layer_set.get(name='geonode:layer_with_dates_in_abstract')
        # this layer has the following dates
        # in abstract: 1901, 1902
        for date in ('1901-01-01', '1902-01-01'):
            self.assertEqual(
                             layer_with_dates_in_abstract.layerdate_set.filter(
                                date=date).filter(type=DATE_DETECTED).count(), 1)

        # test dates #5
        layer_with_html_tag = service.layer_set.get(name='geonode:layer_with_html_tag')
        self.assertEqual(layer_with_html_tag.layerdate_set.filter(date=date).filter(type=DATE_DETECTED).count(), 0)

        # check layer geonode:layer_with_many_dates (private)
        layer_1 = service.layer_set.get(name='geonode:layer_with_many_dates')
        self.assertFalse(layer_1.is_public)

        # test that if creating the service and is already exiting it is not being duplicated
        # create the service
        def create_duplicated_service():
            duplicated_service = Service(
                type='Hypermap:WorldMap',
                url='http://worldmap.harvard.edu/'
            )
            duplicated_service.save()

        self.assertRaises(Exception, create_duplicated_service)


if __name__ == '__main__':
    unittest.main()



import csv
import logging
from optparse import make_option

from django.core.management.base import BaseCommand

from hypermap.aggregator.utils import create_services_from_endpoint, get_sanitized_endpoint

LOGGER = logging.getLogger(__name__)


class Command(BaseCommand):
    help = ("Import a text file containing OWS/Rest endpoints to services.")

    args = 'path [path...]'

    option_list = BaseCommand.option_list + (
        make_option(
            '-c',
            '--column',
            dest="column",
            default=0,
            help="Column index containing endpoints in csv"),
    )

    def handle(self, *args, **options):
        column = int(options.get('column'))
        csv_file = args[0]
        with open(csv_file, 'rb') as f:
            reader = csv.reader(f)
            for row in reader:
                LOGGER.debug(row[column])
                endpoint = get_sanitized_endpoint(row[column])
                create_services_from_endpoint(endpoint)



# -*- coding: utf-8 -*-

"""
Tests for the WMS Service Type.
"""

import unittest

from httmock import with_httmock
import mocks.wms

from hypermap.aggregator.models import Service, Catalog


class TestWMS(unittest.TestCase):

    @with_httmock(mocks.wms.resource_get)
    def test_create_wms_service(self):
        catalog, created = Catalog.objects.get_or_create(
            name="hypermap", slug="hypermap",
            url="search_api"
        )

        # create the service
        service = Service(
            type='OGC:WMS',
            url='http://wms.example.com/ows?',
            catalog=catalog
        )
        service.save()

        # check layer number
        self.assertEqual(service.layer_set.all().count(), 9)

        # check layer geonode:_30river_project1_1 (public)
        layer_0 = service.layer_set.get(name='geonode:_30river_project1_1')
        self.assertEqual(layer_0.title, 'Rivers')
        self.assertTrue(layer_0.is_public)
        self.assertEqual(layer_0.keywords.all().count(), 3)
        self.assertEqual(layer_0.service.srs.all().count(), 2)
        self.assertEqual(layer_0.check_set.all().count(), 1)
        # TODO test layer_0.layerdate_set

        # test that if creating the service and is already exiting it is not being duplicated
        # create the service
        def create_duplicated_service():
            duplicated_service = Service(
                type='OGC:WMS',
                url='http://wms.example.com/ows?',
            )
            duplicated_service.save()

        self.assertRaises(Exception, create_duplicated_service)


if __name__ == '__main__':
    unittest.main()



from django.db import models


class Dynasty(models.Model):
    """
    Dynasty represents different date periods and dynasties to check when mining
    """
    date_range = models.CharField(max_length=255, null=True, blank=True)
    name = models.CharField(max_length=255, null=True, blank=True)

    def __unicode__(self):
        return self.name



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import django_extensions.db.fields


class Migration(migrations.Migration):

    dependencies = [
        ('aggregator', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='layer',
            name='catalogs',
        ),
        migrations.RemoveField(
            model_name='layer',
            name='srs',
        ),
        migrations.AddField(
            model_name='catalog',
            name='url',
            field=models.URLField(help_text=b'Only if remote. URL where the API for the search backend is served.ex: http://localhost:8000/registry/api/search/', max_length=255, null=True, blank=True),
        ),
        migrations.AddField(
            model_name='endpoint',
            name='catalog',
            field=models.ForeignKey(default=1, to='aggregator.Catalog'),
        ),
        migrations.AddField(
            model_name='endpointlist',
            name='catalog',
            field=models.ForeignKey(default=1, to='aggregator.Catalog'),
        ),
        migrations.AddField(
            model_name='endpointlist',
            name='greedy',
            field=models.BooleanField(default=False, editable=False),
        ),
        migrations.AddField(
            model_name='layer',
            name='catalog',
            field=models.ForeignKey(default=1, to='aggregator.Catalog'),
        ),
        migrations.AddField(
            model_name='layer',
            name='csw_last_updated',
            field=models.CharField(default=b'2016-08-27T03:13:11Z', max_length=32, null=True, blank=True),
        ),
        migrations.AddField(
            model_name='layer',
            name='is_monitored',
            field=models.BooleanField(default=True),
        ),
        migrations.AddField(
            model_name='service',
            name='catalog',
            field=models.ForeignKey(default=1, to='aggregator.Catalog'),
        ),
        migrations.AddField(
            model_name='service',
            name='csw_last_updated',
            field=models.CharField(default=b'2016-08-27T03:13:11Z', max_length=32, null=True, blank=True),
        ),
        migrations.AddField(
            model_name='service',
            name='srs',
            field=models.ManyToManyField(to='aggregator.SpatialReferenceSystem', blank=True),
        ),
        migrations.AlterField(
            model_name='catalog',
            name='name',
            field=models.CharField(help_text=b'Display name in UI', max_length=255),
        ),
        migrations.AlterField(
            model_name='catalog',
            name='slug',
            field=django_extensions.db.fields.AutoSlugField(editable=False, populate_from=b'name', blank=True, help_text=b'Leave empty to be populated from name'),
        ),
        migrations.AlterField(
            model_name='endpoint',
            name='endpoint_list',
            field=models.ForeignKey(blank=True, to='aggregator.EndpointList', null=True),
        ),
        migrations.AlterField(
            model_name='endpoint',
            name='url',
            field=models.URLField(max_length=255),
        ),
        migrations.AlterField(
            model_name='layer',
            name='page_url',
            field=models.URLField(max_length=255, null=True, blank=True),
        ),
        migrations.AlterField(
            model_name='layer',
            name='service',
            field=models.ForeignKey(blank=True, to='aggregator.Service', null=True),
        ),
        migrations.AlterField(
            model_name='layer',
            name='type',
            field=models.CharField(max_length=32, choices=[(b'OGC:CSW', b'Catalogue Service for the Web (CSW)'), (b'OGC:WMS', b'Web Map Service (WMS)'), (b'OGC:WMTS', b'Web Map Tile Service (WMTS)'), (b'OSGeo:TMS', b'Tile Map Service (TMS)'), (b'ESRI:ArcGIS:MapServer', b'ArcGIS REST MapServer'), (b'ESRI:ArcGIS:ImageServer', b'ArcGIS REST ImageServer'), (b'Hypermap:WorldMap', b'Harvard WorldMap'), (b'Hypermap:WARPER', b'Mapwarper')]),
        ),
        migrations.AlterField(
            model_name='service',
            name='type',
            field=models.CharField(max_length=32, choices=[(b'OGC:CSW', b'Catalogue Service for the Web (CSW)'), (b'OGC:WMS', b'Web Map Service (WMS)'), (b'OGC:WMTS', b'Web Map Tile Service (WMTS)'), (b'OSGeo:TMS', b'Tile Map Service (TMS)'), (b'ESRI:ArcGIS:MapServer', b'ArcGIS REST MapServer'), (b'ESRI:ArcGIS:ImageServer', b'ArcGIS REST ImageServer'), (b'Hypermap:WorldMap', b'Harvard WorldMap'), (b'Hypermap:WARPER', b'Mapwarper')]),
        ),
        migrations.AlterUniqueTogether(
            name='endpoint',
            unique_together=set([('url', 'catalog')]),
        ),
    ]



import os
from distutils.core import setup
from distutils.command.install import INSTALL_SCHEMES

from hypermap import __version__, __description__


def read(*rnames):
    with open(os.path.join(os.path.dirname(__file__), *rnames)) as ff:
        return ff.read()


def fullsplit(path, result=None):
    """
    Split a pathname into components (the opposite of os.path.join) in a
    platform-neutral way.
    """
    if result is None:
        result = []
    head, tail = os.path.split(path)
    if head == '':
        return [tail] + result
    if head == path:
        return result
    return fullsplit(head, [tail] + result)

for scheme in INSTALL_SCHEMES.values():
    scheme['data'] = scheme['purelib']

packages, data_files = [], []
root_dir = os.path.dirname(__file__)
if root_dir != '':
    os.chdir(root_dir)
hypermap_dir = 'hypermap'

for dirpath, dirnames, filenames in os.walk(hypermap_dir):
    for i, dirname in enumerate(dirnames):
        if dirname.startswith('.'):
            del dirnames[i]
    if '__init__.py' in filenames:
        packages.append('.'.join(fullsplit(dirpath)))
    elif filenames:
        data_files.append([dirpath, [os.path.join(dirpath, f) for f in filenames]])


setup(
    name=__description__,
    version=__version__,
    author='',
    author_email='',
    url='https://github.com/cga-harvard/HHypermap',
    download_url='https://github.com/cga-harvard/HHypermap',
    description='Django Registry by Harvard CGA',
    long_description=(read('README.md')),
    classifiers=[
        'Development Status :: 1 - Planning',
    ],
    license="BSD",
    keywords="hypermap django",
    packages=packages,
    data_files=data_files,
    zip_safe=False,
    install_requires=[
        'amqplib',
        'arcrest',
        'celery',
        'Django>=1.8, <1.9a0',
        'django-debug-toolbar',
        'django-pagination',
        'django-taggit',
        'django-extensions',
        'dj-database-url',
        'django-cache-url',
        'django-basic-authentication-decorator',
        'elasticsearch',
        'pika',
        'pycsw>=2.0.0',
        'flake8',
        'httmock',
        'djmp>=0.2.5',
        'MapProxy>=1.9',
        'djangorestframework',
        'django-celery',
        'isodate',
        'nose',
        'OWSLib',
        'Paver',
        'Pillow',
        'python-memcached',
        'psycopg2',
        'pysolr',
        'pylibmc',
        'requests',
        'webtest',
    ]
)
